# Copyright (C) 1997 Red Hat Software, Inc.
# Use of this software is subject to the terms of the GNU General
# Public License

# This module manages SMB configuration file handling
# These classes are available:
# ConfSMB(Conf):
#  Implements a dictionary of dictionaries of bools, ints, and strings.
#  Toplevel key is a string representing a share
#  Second level keys are literals representing config options.
#  Booleans are set and stored as 0/1, but read in the file as any legal
#  value, including (case insensitive) 0/false/no and 1/true/yes
#  So: turn off browsing home directories with
#     smb['homes'][browseable] = 0


from string import *
from regex import *
from Conf import *
import regsub
import os

# ConfSMB(Conf):
#  Implements a dictionary of dictionaries of bools, ints, and strings.
#  Toplevel key is a string representing a share
#  Second level keys are literals representing config options.
#  Booleans are set and stored as 0/1, but read in the file as any legal
#  value, including (case insensitive) 0/false/no and 1/true/yes
#  So: turn off browsing home directories with
#     smb['homes'][browseable] = 0
class ConfSMB(Conf):
    def __init__(self, filename='/etc/smb.conf'):
	Conf.__init__(self, filename, ';', '[\t ]*=[\t ]*', '=',
                      merge=0, create_if_missing=0)
    def read(self):
        Conf.read(self)
        self.initvars()
    def initvars(self):
        self.vars = {}
        self.rewind()
        while self.findnextcodeline():
            var = self.getfields()
	    if len(var) == 3:
		if not self.vars.has_key(var[1]):
		    self.vars[var[1]] = [[var[0], var[2]]]
		else:
		    self.vars[var[1]].append([var[0], var[2]])
            self.nextline()
        self.rewind()
    def is_stanza_decl(self):
	# return true if the current line is of the form [...]
	# FIXME
	pass
    def find_stanza(self, stanza_name):
	# leave the current line at the first line of the stanza
	# (the first line after the [stanza_name] entry)
	# FIXME
	pass
    def find_entry_in_current_stanza(self, entry_name):
	# leave the current line at the entry_name line or before
	# the [...] line of the next stanza (or the end of the file)
	# if entry_name does not exist.
	# FIXME
	pass
    def __getitem__(self, varname):
        if self.vars.has_key(varname):
            return self.vars[varname]
        else:
            return [[]]
    def __setitem__(self, varname, value):
	if len(value):
            self.vars[varname] = value
	self.rewind()
	for pair in value:
	    if not self.findlinewithfield(2, varname):
		self.fsf()
	    self.setfields([pair[0], varname, pair[1]])
	    self.nextline()
	while self.findlinewithfield(2, varname):
	    self.deleteline()
    def __delitem__(self, varname):
        del self.vars[varname]
	self.rewind()
	while self.findlinewithfield(2, varname):
	    self.deleteline()
    def keys(self):
	# no need to return list in order here, I think.
	return self.vars.keys()
    def has_key(self, key):
	return self.vars.has_key(key)
